/*

    File: parallax offset limit.frag

    Copyright (C) 2000-2009 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
varying vec3 lightVec;
varying vec3 eyeVec;
varying vec2 texCoord;
uniform sampler2D colorMap;
uniform sampler2D normalMap;
uniform float polyThickness;	//I used 0.5, people ues 0.05 ... whatever.  its proportional to one unit in texture space (??)

#define OFFSET_LIMITING

void main (void) {
	
	float lightDistSqr = dot(lightVec,lightVec);
	float lightDist = sqrt(lightDistSqr);
	
	//3% decrease for using division instead of multiplication
	float attenuation = 1.0 / (gl_LightSource[0].constantAttenuation +
								gl_LightSource[0].linearAttenuation * lightDist +
								gl_LightSource[0].quadraticAttenuation * lightDistSqr);

	vec3 unitLightVec = lightVec / lightDist;

	vec3 unitEyeVec = normalize(eyeVec);
	
	vec4 bumpTexel = texture2D(normalMap, texCoord);
	
	//here's where we do our first parallax hack:
	//we can reiterate this upon one condition:
	//each component of the eyeVec.xy times d/dxy bumpTexel.w stays within (-2*unitEyeVec.z/polyThickness, 0)
	//subtract 0.5 from the depth for biasing
#ifndef OFFSET_LIMITING	//similar triangles -- doesnt require normalizing the view vector but we gotta anyways
	vec2 offsetTexCoord = texCoord + polyThickness * (bumpTexel.w - 0.5) * unitEyeVec.xy / unitEyeVec.z;
#else	//offset can be vector no longer than the height vector -- requires normalizing the view vector
	vec2 offsetTexCoord = texCoord + polyThickness * (bumpTexel.w - 0.5) * unitEyeVec.xy;
#endif
	
	//now that we got our offset'd colormap, pull the color and the new bump texel from that instead
	
	vec4 colorMapTexel = texture2D(colorMap, offsetTexCoord);
	
	bumpTexel = texture2D(normalMap, offsetTexCoord);

	vec3 bumpNormal = normalize( bumpTexel.xyz * 2.0 - 1.0);

	vec4 ambientColor = gl_LightSource[0].ambient * gl_FrontMaterial.ambient;

	float diffuseCoeff = max( dot(unitLightVec, bumpNormal), 0.0 );
	vec4 diffuseColor = gl_LightSource[0].diffuse * gl_FrontMaterial.diffuse * diffuseCoeff;	

	float specularCoeff = pow(clamp(dot(reflect(-unitEyeVec, bumpNormal), unitLightVec), 0.0, 1.0), gl_FrontMaterial.shininess);
	vec4 specularColor = gl_LightSource[0].specular * gl_FrontMaterial.specular * specularCoeff;	
	
	gl_FragColor = ambientColor + (diffuseColor*colorMapTexel + specularColor) * attenuation;
}